/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import java.io.Serializable;
import java.util.Collection;

/**
 * The interface for test contexts that are capable of having
 * other nested (child) test contexts.
 *
 * @author    Simon Brown
 */
public interface TestContextContainer extends TestContext, Serializable {

  /**
   * Sets the object responsible for managing test contexts.
   *
   * @param manager   a TestContextManager instance
   */
  void setManager(TestContextManager manager);

  /**
   * Gets the object responsible for managing this and related test contexts.
   *
   * @return  a TestContextManager instance
   */
  public TestContextManager getManager();

  /**
   * Adds a child test context.
   *
   * @param ctx   the new nested/child TestContext instance
   */
  public void add(TestContext ctx);

  /**
   * Gets a collection of the TestContext instances that this
   * container is maintaining.
   *
   * @return  an immutable Collection of TestContext instances
   */
  public Collection getChildren();

  /**
   * Determines whether this container actually has any children.
   *
   * @return  true if it has children, false otherwise
   */
  public boolean hasChildren();

  /**
   * Gets the number of tests that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfTests();

  /**
   * Gets the number of tests that this context represents
   * with the specified status.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfTests(TestContextStatus status);

  /**
   * Gets the number of passing tests that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfPasses();

  /**
   * Gets the number of tests with warnings that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfWarnings();

  /**
   * Gets the number of failing tests that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfFailures();

  /**
   * Gets the number of tests in error that this context represents.
   *
   * @return  the number of tests as an integer
   */
  public int getNumberOfErrors();

}