/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.ant;

// todo: add an output stream property (probably just a file name)

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.tagunit.TagUnitProperties;
import org.tagunit.TestContext;
import org.tagunit.TestContextContainer;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;

/**
 * Ant task to run TagUnit tests automatically.
 *
 * @author    Sam Dalton
 */
public class TagUnitTask extends Task {

  private String url = "";
  private boolean stopOnFail = true;
  private boolean ignoreWarnings = false;
  private Formatter formatter = new ConsoleFormatter();

  /** the file to log the output to */
  private File logFile;

  public void execute() throws BuildException {

    System.out.println("Running tests at " + url);

    // are we ignoring warnings?
    TagUnitProperties.getInstance().setIgnoreWarnings(ignoreWarnings);

    try {
      // open up a URL connection, get the objects back
      boolean hasQuery = false;
      if (url.indexOf("?") > 0) {
        hasQuery = true;
      }
      url = url + (hasQuery ? "&" : "?") + "client=auto";

      URL u = new URL(url);
      InputStream in = u.openStream();
      ObjectInputStream ois = new ObjectInputStream(in);
      TestContextContainer container = (TestContextContainer)ois.readObject();

      // setup the file logging
      if (logFile != null) {
        configLogFile();
      }

      // Display header and summary information
      formatter.displayHeader(container);

      // Check the top level object for failures
      if (container.getStatus().equals(TestContext.FAIL) || container.getStatus().equals(TestContext.WARNING)) {

        // if there were errors
        if (stopOnFail) {
          throw new BuildException();
        }
      }

      if (container.hasChildren()) {
        Collection tests = container.getChildren();
        //todo: perhaps change this to show ALL tests and let the formatter decide what to do...
        displayTests(tests);
      }

      formatter.displaySummary(container);

      ois.close();
    } catch (MalformedURLException e) {
      System.err.println("Please specify a valid URL");
    } catch (IOException e) {
      System.out.println("Can't read from URL");
    } catch (ClassNotFoundException e) {
      // something has gone very wrong
      e.printStackTrace();
    }
  }

  /**
   * Sets up the formatter to log to a file if a log file has been
   * specified.
   *
   * @throws BuildException if any error occurs.  Could happen if the given
   *         file is a directory or if the file cannot be written to.
   */
  private void configLogFile() throws BuildException {
    // check that the log file is not a directory
    if (logFile.exists() && logFile.isDirectory()) {
      throw new BuildException("The log file cannot be a directory.");
    } else {
      // create the file (to make sure it can be written)
      try {
        logFile.createNewFile();

        // logFile okay, make formatter log to the file
        formatter.setOutputStream(new FileOutputStream(logFile));
      } catch (FileNotFoundException fnfe) {
        fnfe.printStackTrace();
      } catch (IOException ioe) {
        throw new BuildException("The log file could not be created.");
      }
    }
  }

  private void displayTests(Collection tests) {

    // make sure this only displays warnings and errors
    Iterator it = tests.iterator();

    TestContext context = null;
    while (it.hasNext()) {
      context = (TestContext)it.next();
      if (context instanceof TestContextContainer) {
        TestContextContainer container = (TestContextContainer)context;

        if (container.hasChildren()) {
          displayTests(container.getChildren());

        }
        formatter.displayTest(context);

      } else if (context.getStatus().equals(TestContext.FAIL) || context.getStatus().equals(TestContext.WARNING)) {
        formatter.displayTest(context);
      }
    }
  }

  public void setUrl(String url) {
    this.url = url;
  }

  public void setStopOnFail(boolean stopOnFail) {
    this.stopOnFail = stopOnFail;
  }

  /**
   * Sets the path to the file to log the output to.
   *
   * <P> Optional attribute:  if it is not specified, then the
   * output is simply echoed to the stdout (console typically).
   * If is specified, an attempt is made to create the specified
   * file and the output is written to the file.
   *
   * @param file the path to the log file.
   */
  public void setLogFile(File file) {
    this.logFile = file;
  }

  /**
   * Sets whether warnings should be ignored or not.
   *
   * @param ignoreWarnings    true if warnings should be ignored,
   *                          false otherwise
   */
  public void setIgnoreWarnings(boolean ignoreWarnings) {
    this.ignoreWarnings = ignoreWarnings;
  }

  public void setFormatter(String formatter) {
    try {
      this.formatter = (Formatter)this.getClass().getClassLoader().loadClass(formatter).newInstance();
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();

    } catch (ClassNotFoundException e) {
      e.printStackTrace();

    }
  }

}
