/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.taginfo;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.*;

import org.tagunit.taginfo.jsp11.Jsp11TagLibraryDescriptorParser;
import org.tagunit.taginfo.jsp12.Jsp12TagLibraryDescriptorParser;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.xml.sax.*;

/**
 * A factory for creating TagLibraryInfo objects - in-memory representations
 * of tag library descriptor files.
 *
 * todo   refactor this
 *
 * @author Simon Brown
 */
public final class TagLibraryInfoFactory {

  /** the public ID of the JSP taglib 1.1 DTD */
  public static final String TAGLIB_DTD_PUBLIC_ID_11 =
      "-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.1//EN";

  /** the public ID of the JSP taglib 1.2 DTD */
  public static final String TAGLIB_DTD_PUBLIC_ID_12 =
      "-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.2//EN";

  /** the system ID of the JSP taglib 2.0 XML schema */
  public static final String TAGLIB_20_SYSTEM_ID =
      "http://java.sun.com/xml/ns/j2ee/web-jsptaglibrary_2_0.xsd";

  /** the system ID of the J2EE 1.4 XML schema */
  public static final String J2EE_14_SYSTEM_ID =
      "http://java.sun.com/xml/ns/j2ee/j2ee_1_4.xsd";

  /** the system ID of the XML schema */
  public static final String XML_SYSTEM_ID =
      "http://www.w3.org/2001/xml.xsd";

  /** the system ID of the web services client XML schema */
  public static final String WEB_SERVICES_CLIENT_SYSTEM_ID =
      "http://www.ibm.com/webservices/xsd/j2ee_web_services_client_1_1.xsd";

  //private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
  //private static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
  //private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
  //private static final String TAGLIB_20_SCHEMA = "http://java.sun.com/xml/ns/j2ee/web-jsptaglibrary_2_0.xsd";

  /** the document builder used within this tag for parsing XML documents */
  private DocumentBuilder builder;

  /**
   * Default, no args constructor.
   */
  public TagLibraryInfoFactory() {
    try {
      // create a factory and builder - an abstraction for an XML parser
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating(true);
      factory.setNamespaceAware(true);
      factory.setIgnoringElementContentWhitespace(true);
      factory.setIgnoringComments(true);
      //factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
      //factory.setAttribute(JAXP_SCHEMA_SOURCE, TAGLIB_20_SCHEMA);
      this.builder = factory.newDocumentBuilder();
      builder.setEntityResolver(LocalEntityResolver.getInstance());
      builder.setErrorHandler(new ErrorHandler() {
        public void warning(SAXParseException e) throws SAXException {
          System.out.println("Warning : " + e.getMessage());
          throw e;
        }

        public void error(SAXParseException e) throws SAXException {
          System.out.println("Error : " + e.getMessage());
          throw e;
        }

        public void fatalError(SAXParseException e) throws SAXException {
          System.out.println("Fatal : " + e.getMessage());
          throw e;
        }
      });
    } catch (ParserConfigurationException pce) {
      pce.printStackTrace();
    }
  }

  /**
   * Gets information about the tag library from the specified InputStream.
   *
   * @param in    a java.io.InputStream instance
   * @return      a TagLibraryInfo instance
   */
  public TagLibraryInfo getTagLibraryInfo(InputStream in)
      throws TagLibraryNotSupportedException, TagLibraryInfoParseException {

    if (in == null) {
      throw new TagLibraryInfoParseException("No TLD found");
    }

    try {
      return getTagLibraryInfo(builder.parse(in));
    } catch (IOException ioe) {
      ioe.printStackTrace();
      throw new TagLibraryInfoParseException(ioe.getMessage());
    } catch (SAXException se) {
      throw new TagLibraryInfoParseException(se.getMessage());
    }
  }

  /**
   * Gets information about the tag library from the specified Document.
   *
   * @param doc       an XML Document
   * @return      a TagLibraryInfo instance
   */
  private TagLibraryInfo getTagLibraryInfo(Document doc)
      throws TagLibraryNotSupportedException, TagLibraryInfoParseException {

    // find out which version of the JSP spec the taglib supports
    DocumentType docType = doc.getDoctype();

    // and now create the appropriate factory to read the document
    if (docType.getPublicId().equals(TAGLIB_DTD_PUBLIC_ID_11)) {
      TagLibraryDescriptorParser parser = new Jsp11TagLibraryDescriptorParser();
      return parser.parse(doc);
    } else if (docType.getPublicId().equals(TAGLIB_DTD_PUBLIC_ID_12)) {
      TagLibraryDescriptorParser parser = new Jsp12TagLibraryDescriptorParser();
      return parser.parse(doc);
    } else {
      throw new TagLibraryNotSupportedException("The JSP version of the tag library ("
          + docType.getPublicId() + ") is currently not supported.");
    }
  }

  /**
   * An entity resolver that is used to resolve references to the DTDs/schemas
   * locally rather than going out onto the internet.
   */
  static class LocalEntityResolver implements EntityResolver {

    private static final EntityResolver instance = new LocalEntityResolver();
    private Map localResources;

    private LocalEntityResolver() {
      localResources = new HashMap();
      localResources.put(TAGLIB_DTD_PUBLIC_ID_11, "/web-jsptaglibrary_1_1.dtd");
      localResources.put(TAGLIB_DTD_PUBLIC_ID_12, "/web-jsptaglibrary_1_2.dtd");
      localResources.put(TAGLIB_20_SYSTEM_ID, "/web-jsptaglibrary_2_0.xsd");
      localResources.put(J2EE_14_SYSTEM_ID, "/j2ee_1_4.xsd");
      localResources.put(XML_SYSTEM_ID, "/xml.xsd");
      localResources.put(WEB_SERVICES_CLIENT_SYSTEM_ID, "/j2ee_web_services_client_1_1.xsd");
    }

    public static EntityResolver getInstance() {
      return instance;
    }

    /**
     * Resolves a DTD, returning an InputSource.
     *
     * @param publicId    the public ID of the DTD
     * @param systemId    the system ID
     * @return  an InputSource for the DTD, or null if the default behaviour
     *          should be used
     */
    public InputSource resolveEntity(String publicId, String systemId) {

      //System.out.println("PUBLIC ID = " + publicId);
      //System.out.println("SYSTEM ID = " + systemId);

      if (publicId != null) {
        String resource = (String)localResources.get(publicId);
        InputStream in = getClass().getResourceAsStream(resource);
        return new InputSource(in);
      } else if (systemId != null) {
        String resource = (String)localResources.get(systemId);
        InputStream in = getClass().getResourceAsStream(resource);
        return new InputSource(in);
      } else {
        return null;
      }
    }

  }

}

