/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import org.tagunit.*;
import org.tagunit.taginfo.TagAttributeInfo;
import org.tagunit.taginfo.TagInfo;

/**
 * Wraps up the automatic tests for a tag handler.
 *
 * @author    Simon Brown
 */
public class AttributeTestPackage extends AbstractTagTestPackage {

  private String name;
  private String type;
  private boolean required;
  private boolean rtexprvalue;

  /**
   * Creates a new test package for the specified tag.
   *
   * @param tagInfo   a TagInfo instance
   */
  public AttributeTestPackage(TagInfo tagInfo, String name, String type,
                              boolean required, boolean rtexprvalue) {
    super(tagInfo);
    this.name = name;
    this.type = type;
    this.required = required;
    this.rtexprvalue = rtexprvalue;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    TagAttributeInfo attr = getTagInfo().getAttribute(name);
    TestContext ctx;

    ctx = new DefaultTestContext("Attribute named " + name + " exists");
    testContext.add(ctx);
    boolean result = (attr != null);
    if (result) {
      ctx.setStatus(TestContext.PASS);
    } else {
      ctx.setStatus(TestContext.FAIL);
      ctx.setMessage("Attribute named " + name + " does not exist.");

      // since the attribute doesn't exist, we can't perform any more tests!
      return;
    }

    if (type != null && type.length() > 0) {
      testType(attr, testContext);
    }

    testRequired(attr, testContext);
    testRequestTimeExpression(attr, testContext);
  }

  private void testType(TagAttributeInfo attr, TestContextContainer testContext) {
    TestContext ctx;

    ctx = new DefaultTestContext("Attribute named " + name + " is of type " + type);
    testContext.add(ctx);

    boolean result = type.equals(attr.getType());
    if (result) {
      ctx.setStatus(TestContext.PASS);
    } else {
      ctx.setStatus(TestContext.FAIL);
      ctx.setMessage("Attribute named " + name + " is of type " + attr.getType());
    }
  }

  private void testRequired(TagAttributeInfo attr, TestContextContainer testContext) {
    TestContext ctx;

    if (required) {
      ctx = new DefaultTestContext("Attribute named " + name + " is marked as required");
    } else {
      ctx = new DefaultTestContext("Attribute named " + name + " is marked as required");
    }
    testContext.add(ctx);

    boolean result = (required == attr.isRequired());
    if (result) {
      ctx.setStatus(TestContext.PASS);
    } else {
      ctx.setStatus(TestContext.FAIL);

      if (attr.isRequired()) {
        ctx.setMessage("Attribute named " + name + " is marked as required.");
      } else {
        ctx.setMessage("Attribute named " + name + " is marked as optional.");
      }
    }
  }

  private void testRequestTimeExpression(TagAttributeInfo attr, TestContextContainer testContext) {
    TestContext ctx;

    if (rtexprvalue) {
      ctx = new DefaultTestContext("Attribute named " + name + " accepts request time expressions");
    } else {
      ctx = new DefaultTestContext("Attribute named " + name + " doesn't accept request time expressions");
    }
    testContext.add(ctx);

    boolean result = (rtexprvalue == attr.canAcceptRequestTimeExpressions());
    if (result) {
      ctx.setStatus(TestContext.PASS);
    } else {
      ctx.setStatus(TestContext.FAIL);

      if (attr.canAcceptRequestTimeExpressions()) {
        ctx.setMessage("Attribute named " + name + " accepts runtime expressions.");
      } else {
        ctx.setMessage("Attribute named " + name + " doesn't accept runtime expressions.");
      }
    }
  }

  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public String getName() {
    return "Attribute named " + this.name + " (required = " + required + ")";
  }

}