/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.Iterator;

import org.tagunit.*;
import org.tagunit.taginfo.TagAttributeInfo;
import org.tagunit.taginfo.TagInfo;
import org.tagunit.util.ExceptionUtils;

/**
 * Wraps up the automatic tests for a tag handler.
 *
 * @author    Simon Brown
 */
public class AutomaticTagHandlerTestPackage extends AbstractTestPackage {

  /** information about the tag being tested */
  private TagInfo tagInfo;

  /**
   * Creates a new test package for the specified tag.
   *
   * @param tagInfo   a TagInfo instance
   */
  public AutomaticTagHandlerTestPackage(TagInfo tagInfo) {
    this.tagInfo = tagInfo;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    // these tests are automatic so let's flag them that way
    System.out.print(" - Running automatic tests for " + tagInfo.getName());

    TestContextContainer tagHandlerTests = new DefaultTestContextContainer("Automatic tag handler tests");
    testContext.add(tagHandlerTests);
    testContext.setType(TestContext.AUTOMATIC_TEST);

    long start;
    long end;

    start = System.currentTimeMillis();
    TestContext ctx = new DefaultTestContext("Tag handler class is loadable");
    tagHandlerTests.add(ctx);
    testClassLoadable(ctx, tagInfo.getTagHandlerClass());

    // only do the next batch of tests if that first one worked
    if (ctx.getResult()) {
      ctx = new DefaultTestContext("Tag handler class can be instantiated");
      tagHandlerTests.add(ctx);
      testClassInstantiable(ctx, tagInfo.getTagHandlerClass());

      ctx = new DefaultTestContext("Tag handler class implements javax.servlet.jsp.tagext.Tag interface");
      tagHandlerTests.add(ctx);
      testClassImplementsInterface(ctx,
          tagInfo.getTagHandlerClass(), javax.servlet.jsp.tagext.Tag.class);

      if (tagInfo.getTagExtraInfoClass() != null) {
        ctx = new DefaultTestContext("TEI class is loadable");
        tagHandlerTests.add(ctx);
        testClassLoadable(ctx, tagInfo.getTagExtraInfoClass());

        ctx = new DefaultTestContext("TEI handler class can be instantiated");
        tagHandlerTests.add(ctx);
        testClassInstantiable(ctx, tagInfo.getTagExtraInfoClass());

        ctx = new DefaultTestContext("TEI class extends javax.servlet.jsp.tagext.TagExtraInfo class");
        tagHandlerTests.add(ctx);
        testClassImplementsInterface(ctx,
            tagInfo.getTagExtraInfoClass(), javax.servlet.jsp.tagext.TagExtraInfo.class);
      }

      if (tagHandlerTests.getResult()) {
        testAttributes(tagHandlerTests);
      }
    }

    end = System.currentTimeMillis();
    tagHandlerTests.setTime(end - start);
    System.out.println(" (" + tagHandlerTests.getTime() + " ms)");
  }

  /**
   * Tests whether the tag handler class is available to be loaded by the
   * classloader.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  private void testClassLoadable(TestContext testContext, String className) {
    // can the tag handler class be loaded from the classpath?
    try {
      getClass(className);
      testContext.setStatus(TestContext.PASS);
    } catch (ClassNotFoundException cnfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class " + className + " not found on the classpath for this web application.");
    } catch (NoClassDefFoundError ncdfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class referenced by " + className +
          " not found on the classpath for this web application (" +
          ncdfe.getMessage() + ")");
    } catch (Exception e) {
      testContext.setStatus(TestContext.ERROR);
      testContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
    }
  }

  /**
   * Tests whether the tag handler class is available to be loaded by the
   * classloader.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  private void testClassInstantiable(TestContext testContext, String className) {
    // can the tag handler class be loaded from the classpath?
    try {
      Class c = getClass(className);
      int mods = c.getModifiers();

      if (Modifier.isAbstract(mods)) {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Class " + className + " is abstract.");
      } else if (Modifier.isInterface(mods)) {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Class " + className + " is an interface.");
      } else {
        testContext.setStatus(TestContext.PASS);
      }

    } catch (ClassNotFoundException cnfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class " + className + " not found on the classpath for this web application.");
    } catch (NoClassDefFoundError ncdfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class referenced by " + className +
          " not found on the classpath for this web application (" +
          ncdfe.getMessage() + ")");
    } catch (Exception e) {
      testContext.setStatus(TestContext.ERROR);
      testContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
    }
  }

  /**
   * Tests whether the tag handler class implements javax.servlet.jsp.tagext.Tag.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  private void testClassImplementsInterface(TestContext testContext, String className, Class anInterface) {
    // does the tag handler class implement the Tag interface?
    try {
      Class c = Class.forName(className);

      if (!anInterface.isAssignableFrom(c)) {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Class " + className + " doesn't implement " + anInterface.getName() + " interface");
      } else {
        testContext.setStatus(TestContext.PASS);
      }
    } catch (ClassNotFoundException cnfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class " + className + " not found on the classpath for this web application.");
    } catch (NoClassDefFoundError ncdfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class referenced by " + className +
          " not found on the classpath for this web application (" +
          ncdfe.getMessage() + ")");
    } catch (Exception e) {
      testContext.setStatus(TestContext.ERROR);
      testContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
    }
  }


  /**
   * Tests whether the tag handler class has setters for each attribute.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  private void testAttributes(TestContextContainer testContext) {

    TestContext ctx;

    try {
      Class c = Class.forName(tagInfo.getTagHandlerClass());
      Method[] methods = c.getMethods();

      Collection attributes = tagInfo.getAttributes();
      TagAttributeInfo attr;

      Iterator it = attributes.iterator();
      while (it.hasNext()) {
        attr = (TagAttributeInfo)it.next();

        ctx = new DefaultTestContext("Tag handler class has setter for \"" + attr.getName() + "\" attribute");
        testContext.add(ctx);

        // work out the name of the setter method for the attribute
        // (based upon the JavaBeans specification)
        String setterName = "set" + attr.getName().substring(0, 1).toUpperCase()
            + attr.getName().substring(1);

        boolean found = false;
        for (int i = 0; i < methods.length; i++) {
          if (methods[i].getName().equals(setterName)) {
            if (methods[i].getParameterTypes().length == 1) {

              // now that we've found it, check the parameter type is assignable
              Class parameterType = methods[i].getParameterTypes()[0];
              Class attributeType = getClass(attr.getType());

              if (parameterType.isAssignableFrom(attributeType)) {
                found = true;
              } else if (attributeType == String.class) {
                found = true;
              }
            }
          }
        }


        if (found) {
          ctx.setStatus(TestContext.PASS);
        } else {
          ctx.setStatus(TestContext.FAIL);
          ctx.setMessage("Tag handler class doesn't provide a setter method for \""
              + attr.getName() + "\" attribute");
        }

        // and also, check whether the type of the attribute is loadable
        ctx = new DefaultTestContext("Class for \"" + attr.getName() + "\" attribute is loadable");
        testContext.add(ctx);
        testClassLoadable(ctx, attr.getType());
      }
    } catch (Exception e) {
      testContext.setStatus(TestContext.ERROR);
      testContext.setMessage(ExceptionUtils.getStackTraceAsString(e));
    } catch (NoClassDefFoundError ncdfe) {
      testContext.setStatus(TestContext.FAIL);
      testContext.setMessage("Class referenced by " + tagInfo.getTagHandlerClass() +
          " not found on the classpath for this web application (" +
          ncdfe.getMessage() + ")");
    }

  }

  private Class getClass(String className) throws ClassNotFoundException {
    if (className.equals("byte")) {
      return Byte.TYPE;
    } else if (className.equals("short")) {
      return Short.TYPE;
    } else if (className.equals("char")) {
      return Character.TYPE;
    } else if (className.equals("int")) {
      return Integer.TYPE;
    } else if (className.equals("long")) {
      return Long.TYPE;
    } else if (className.equals("float")) {
      return Float.TYPE;
    } else if (className.equals("double")) {
      return Double.TYPE;
    } else if (className.equals("boolean")) {
      return Boolean.TYPE;
    } else {
      return Class.forName(className);
    }
  }

  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public String getName() {
    return "TagUnit automatic tag handler tests";
  }

}