/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import java.util.*;

import org.tagunit.TestContextContainer;

/**
 * This class provides a way to build user-defined test packages.
 *
 * @author    Simon Brown
 */
public abstract class CustomTestPackage extends AbstractTestPackage {

  /** the name of the test package */
  private String name;

  /** a map that can be used to store parameters */
  private HashMap parameters;

  /**
   * Creates a new test package for the specified tag.
   *
   * @param name    the name of the test package
   */
  public CustomTestPackage(String name) {
    this.name = name;
  }

  /**
   * Sets the parameters for this test package.
   *
   * @param params    a Map containing the parameters
   */
  public void setParameters(HashMap params) {
    this.parameters = params;
  }

  /**
   * Gets the collection of parameter names.
   *
   * @return  a Collection of parameter names (Strings)
   */
  public Collection getParameterNames() {
    return Collections.unmodifiableCollection(parameters.keySet());
  }

  /**
   * Gets the collection of parameter values.
   *
   * @return  a Collection of parameter values (Objects)
   */
  public Collection getParameterValues() {
    return Collections.unmodifiableCollection(parameters.values());
  }

  /**
   * Gets the value of a named parameter.
   *
   * @param name    the name of the parameter
   * @return  the value of the named paramater as an Object
   */
  public Object getParameter(String name) {
    return parameters.get(name);
  }

  /**
   * Runs the tests that are part of this test package. <b>User defined test
   * packages should override this method.</b>
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public abstract void executeTest(TestContextContainer testContext);

  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public final String getName() {
    return "Custom test package - " + this.name;
  }

}