/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit.test;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.Map;

import javax.servlet.jsp.PageContext;

import org.tagunit.TestContextContainer;
import org.tagunit.TestContext;

/**
 * Asserts that a named (and optionally, scoped) page context attribute
 * does exists.
 *
 * @author    Simon Brown
 */
public class PageContextAttributeTestPackage extends AbstractTestPackage {

  private String name;
  private String property;
  private String type;
  private Object value;
  private String scope;

  /**
   * Creates a new test package for the specified tag.
   */
  public PageContextAttributeTestPackage(String name, String property, String type, Object value, String scope) {
    this.name = name;
    this.property = property;
    this.type = type;
    this.value = value;
    this.scope = scope;
  }

  /**
   * Runs the tests that are part of this test package.
   *
   * @param testContext   the TestContext to which these tests are to be run
   */
  public void executeTest(TestContextContainer testContext) {
    Object variable = getAttribute();

    if (property != null) {
      if (variable instanceof Map) {
        variable = ((Map)variable).get(property);
      } else {
        try {
          PropertyDescriptor pd = new PropertyDescriptor(property, (Class.forName(type)));
          Method m = pd.getReadMethod();
          variable = m.invoke(variable, null);
        } catch (Exception e) {
          testContext.setMessage(e.getMessage());
          testContext.setStatus(TestContext.FAIL);
          return;
        }
      }
    }

    // first of all find the named attribute
    if (variable == null) {
      testContext.setMessage("PageContext attribute named " + name + " does not exist.");
      testContext.setStatus(TestContext.FAIL);

      // well the variable couldn't be found so we can't perform any more tests on it!
      return;
    }

    try {
      boolean result = (Class.forName(type).isAssignableFrom(variable.getClass()));

      if (result) {
        testContext.setStatus(TestContext.PASS);
      } else {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Type of variable is " + variable.getClass());
      }
    } catch (ClassNotFoundException cnfe) {
      cnfe.printStackTrace();
    }

    // and finally check the value, using the equals() method
    if (value != null) {
      boolean result = variable.equals(value);

      if (result) {
        testContext.setStatus(TestContext.PASS);
      } else {
        testContext.setStatus(TestContext.FAIL);
        testContext.setMessage("Value of variable is " + variable);
      }
    }
  }

  /**
   * Helper method to find a named, scoped attribute.
   *
   * @return  an Object attribute
   */
  private Object getAttribute() {
    if (scope == null) {
      // @todo    decide whether this should be page scope or any
      return getPageContext().findAttribute(name);
    } else if (scope.equalsIgnoreCase("request")) {
      return getPageContext().getAttribute(name, PageContext.REQUEST_SCOPE);
    } else if (scope.equalsIgnoreCase("page")) {
      return getPageContext().getAttribute(name, PageContext.PAGE_SCOPE);
    } else if (scope.equalsIgnoreCase("session")) {
      return getPageContext().getAttribute(name, PageContext.SESSION_SCOPE);
    } else if (scope.equalsIgnoreCase("application")) {
      return getPageContext().getAttribute(name, PageContext.APPLICATION_SCOPE);
    } else {
      // @todo    decide whether this should be page scope or any
      return getPageContext().findAttribute(name);
    }
  }

  /**
   * Gets the name of this test package.
   *
   * @return  the name of this test package as a String
   */
  public String getName() {
    if (scope == null) {
      return "Existence of PageContext attribute named " + name + " (type=" + type + ")";
    } else {
      return "Existence of PageContext attribute named " + name + " (type=" + type + ", scope=" + scope + ")";
    }
  }

}