/*
 * Copyright (c) 2002/2003, Simon Brown
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   - Neither the name of TagUnit nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tagunit;

import junit.framework.TestCase;
import org.tagunit.taginfo.TagInfo;

/**
 * Test cases for the TestContextManager class.
 *
 * @author      Simon Brown
 */
public class TestContextManagerTest extends TestCase {

  private TestContextManager manager;

  public void setUp() {
    manager = new TestContextManager(new DefaultTestContextContainer());
  }

  /**
   * Tests the initial values of the object after construction.
   */
  public void testConstruction() {
    assertNotNull(manager.getRootTestContext());

    TestContextContainer root = new DefaultTestContextContainer();
    manager = new TestContextManager(root);
    assertEquals(root, manager.getRootTestContext());
  }

  /**
   * Tests the setters and getters.
   **/
  public void testSettersAndGetters() {
    TestContextContainer root = new DefaultTestContextContainer();
    manager.setRootTestContext(root);
    assertEquals(root, manager.getRootTestContext());
  }

  /**
   * Tests that test contexts can be stacked correctly.
   **/
  public void testSimpleStacking() {
    TestContextContainer root = new DefaultTestContextContainer();
    manager.setRootTestContext(root);

    assertEquals(root, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());

    TestContext context = new DefaultTestContext();
    manager.startTestContext(context);
    assertEquals(context, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());

    manager.endTestContext(context);
    assertEquals(root, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());
  }

  /**
   * Tests that nested test contexts can be stacked correctly.
   **/
  public void testNestedStacking() {
    TestContextContainer root = new DefaultTestContextContainer();
    manager.setRootTestContext(root);

    assertEquals(root, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());

    TestContextContainer container = new DefaultTestContextContainer();
    TestContext context = new DefaultTestContext();

    manager.startTestContext(container);
    assertEquals(container, manager.getCurrentTestContext());
    assertEquals(container, manager.getCurrentTestContextContainer());

    manager.startTestContext(context);
    assertEquals(context, manager.getCurrentTestContext());
    assertEquals(container, manager.getCurrentTestContextContainer());

    manager.endTestContext(context);
    assertEquals(container, manager.getCurrentTestContext());
    assertEquals(container, manager.getCurrentTestContextContainer());

    manager.endTestContext(container);
    assertEquals(root, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());
  }

  /**
   * Tests that tag library and tag test contexts can be stacked correctly.
   **/
  public void testGetTagAndTagLibraryTestContext() {
    TestContextContainer root = new DefaultTestContextContainer();
    manager.setRootTestContext(root);

    TagLibraryTestContext tagLibraryContext = new TagLibraryTestContext();
    TagTestContext tagContext = new TagTestContext();
    TagInfo tagInfo = new TagInfo();
    tagContext.setTagInfo(tagInfo);
    tagInfo.setName("This is the name");

    manager.startTestContext(tagLibraryContext);
    assertEquals(tagLibraryContext, manager.getCurrentTestContext());
    assertEquals(tagLibraryContext, manager.getCurrentTestContextContainer());
    assertEquals(tagLibraryContext, manager.getTagLibraryTestContext());
    assertEquals(null, manager.getTagTestContext());

    manager.startTestContext(tagContext);
    assertEquals(tagContext, manager.getCurrentTestContext());
    assertEquals(tagContext, manager.getCurrentTestContextContainer());
    assertEquals(tagLibraryContext, manager.getTagLibraryTestContext());
    assertEquals(tagContext, manager.getTagTestContext());

    manager.endTestContext(tagContext);
    assertEquals(tagLibraryContext, manager.getCurrentTestContext());
    assertEquals(tagLibraryContext, manager.getCurrentTestContextContainer());
    assertEquals(tagLibraryContext, manager.getTagLibraryTestContext());
    assertEquals(null, manager.getTagTestContext());

    manager.endTestContext(tagLibraryContext);
    assertEquals(root, manager.getCurrentTestContext());
    assertEquals(root, manager.getCurrentTestContextContainer());
    assertEquals(null, manager.getTagLibraryTestContext());
    assertEquals(null, manager.getTagTestContext());
  }

}